@@ -5,27 +5,65 @@
  *
  * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA  02111-1307, USA.
  */
 
-#include "gnome-print-pango.h"
+#include "config.h"
 #define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
 #include <gtk/gtktextlayout.h>
+#include "gnome-print-pango.h"
 
-void gnome_print_pango_layout_print (GnomePrintContext *gpc, PangoLayout* pl) {
+#ifdef GP_HAS_PANGO
+#	include <libgnomeprint/gnome-print-pango.h>
+#	define TEXT_ZOOM_FACTOR	(4. / 3.)
+
+void pango_layout_print (GnomePrintContext *gpc, PangoLayout* pl)
+{
+	PangoAttrList *attrs = pango_layout_get_attributes (pl);
+	PangoLayout *layout = (PangoLayout*) gnome_print_pango_create_layout (gpc);
+	const char *text = pango_layout_get_text (pl);
+	int w1, w2, h1, h2, l;
+	double zoom, extra_space;
+	l = strlen (text);
+	pango_layout_set_text (layout, text, l);
+	pango_layout_set_attributes (layout, attrs);
+	pango_layout_get_size (pl, &w1, &h1);
+	pango_layout_get_size (layout, &w2, &h2);
+	zoom = (double) h1 / (double) h2;
+	extra_space = w1 / zoom - w2;
+	if (l > 1) {
+		PangoAttribute *pa;
+		PangoAttrList *pal = pango_attr_list_copy (attrs);
+		pa = pango_attr_letter_spacing_new ((int) extra_space / l);
+		pa->start_index = 0;
+		pa->end_index = l;
+		pango_attr_list_insert (pal, pa);
+		pango_layout_set_attributes (layout, pal);
+	}
+	gnome_print_gsave (gpc);
+	gnome_print_scale (gpc, zoom, -zoom); /* not so good. Why is zoom necessary to get the same size ? Have a look at resolution */
+	gnome_print_moveto (gpc, 0., 0.);
+	gnome_print_pango_layout (gpc, layout);
+	gnome_print_grestore (gpc);
+	g_object_unref (layout);
+}
+
+#else
+
+void gpc_print_pango_layout_print (GnomePrintContext *gpc, PangoLayout* pl) {
 	gint i, top, bottom;
 	GnomeFont *font;
 	GnomeFontFace *face;
@@ -48,6 +86,7 @@ void gnome_print_pango_layout_print (GnomePrintContext *gpc, PangoLayout* pl) {
 	GtkTextAppearance* appearance;
 	int underline_type;
 	PangoAlignment align = pango_layout_get_alignment (pl);
+	double zoom;
 	
 	pango_layout_get_extents (pl, NULL, &logical_rect);
 	gnome_print_gsave (gpc);
@@ -189,10 +228,11 @@ void gnome_print_pango_layout_print (GnomePrintContext *gpc, PangoLayout* pl) {
 			gnome_glyphlist_glyph (glyph_list, item->glyphs->glyphs[i].glyph);
 		}
 		gnome_glyphlist_bbox (glyph_list, scale, 0, &art_rect);
-		slant = (ink_rect.width / PANGO_SCALE)> (art_rect.x1 - art_rect.x0);
+		slant = (pango_font_description_get_style (desc) != PANGO_STYLE_NORMAL) && !gnome_font_face_is_italic (face);
+		zoom = ((double) ink_rect.height) / PANGO_SCALE / (art_rect.y1 - art_rect.y0);
 		if (item->glyphs->num_glyphs > 1) {
-			space = (((double) ink_rect.width / PANGO_SCALE - art_rect.x1 + art_rect.x0) - (slant)? top * 0.3: 0.)
-						/ (item->glyphs->num_glyphs - 1);
+			space = (((double) ink_rect.width / PANGO_SCALE / zoom - art_rect.x1 + art_rect.x0) - ((slant)? top * 0.15 / zoom: 0.))
+						/ item->glyphs->num_glyphs;
 			gnome_glyphlist_unref (glyph_list);
 			glyph_list = gnome_glyphlist_new ();
 			gnome_print_moveto (gpc, logical_rect.x / PANGO_SCALE,
@@ -208,11 +248,11 @@ void gnome_print_pango_layout_print (GnomePrintContext *gpc, PangoLayout* pl) {
 		if (slant) {
 		/*simulate slanted font*/
 			double matrix [6] = {
-				 1.,
-				0., .15, -1., .0, .0};
+				 zoom,
+				0., .15 * zoom, -zoom, .0, .0};
 			gnome_print_concat (gpc, matrix);
 		} else
-			gnome_print_scale(gpc, 1., -1.);
+			gnome_print_scale(gpc, zoom, -zoom);
 		gnome_print_glyphlist (gpc, glyph_list);
 		gnome_print_grestore(gpc);
 		gnome_glyphlist_unref (glyph_list);
@@ -220,3 +260,4 @@ void gnome_print_pango_layout_print (GnomePrintContext *gpc, PangoLayout* pl) {
 	pango_layout_iter_free (iter);
 	gnome_print_grestore (gpc);
 }
+#endif
