@@ -121,6 +121,7 @@ bool gcpTextObject::SaveNode(xmlDocPtr xml, xmlNodePtr node)
 bool gcpTextObject::Load(xmlNodePtr node)
 {
 	char* tmp, *endptr;
+	bool result;
 	xmlNodePtr child;
 	tmp = (char*)xmlGetProp(node, (xmlChar*)"id");
 	if (tmp)
@@ -132,13 +133,15 @@ bool gcpTextObject::Load(xmlNodePtr node)
 	tmp = (char*)xmlGetProp(node, (xmlChar*)"x");
 	if (!tmp) return false;
 	m_x = strtod(tmp, &endptr);
+	result = *endptr;
 	xmlFree(tmp);
-	if (*endptr) return false;
+	if (result) return false;
 	tmp = (char*)xmlGetProp(node, (xmlChar*)"y");
 	if (!tmp) return false;
 	m_y = strtod(tmp, &endptr);
+	result = *endptr;
 	xmlFree(tmp);
-	if (*endptr) return false;
+	if (result) return false;
 	return true;
 }
 
