@@ -775,6 +775,7 @@ void gcpView::OnSelectAll()
 void gcpView::ExportImage (const char* filename, const char* type)
 {
 	ArtDRect rect;
+	m_pData->ShowSelection (false);
 	m_pData->GetObjectBounds (m_pDoc, &rect);
 	int w = (int) (ceil (rect.x1) - floor (rect.x0)), h = (int) (ceil (rect.y1) - floor (rect.y0));
 	if (!strcmp (type, "eps")) {
@@ -800,11 +801,59 @@ void gcpView::ExportImage (const char* filename, const char* type)
 		char *command_line = g_strconcat ("ps2eps ", tmpname, NULL);
 		g_spawn_command_line_sync (command_line, NULL, NULL, NULL, NULL);
 		g_free (command_line);
-		command_line = g_strconcat ("mv -f ", tmpname, ".eps ", filename);
+		command_line = g_strconcat ("mv -f ", tmpname, ".eps ", filename, NULL);
 		g_spawn_command_line_async (command_line, NULL);
 		remove (tmpname);
 		g_free (tmpname);
 		g_free (command_line);
+	} else if (!strcmp (type, "svg")) {
+		xmlDocPtr doc = xmlNewDoc ((const xmlChar*)"1.0");
+		char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+		char *buf;
+		setlocale (LC_NUMERIC, "C");
+		xmlNewDtd (doc,
+			   (const xmlChar*)"svg", (const xmlChar*)"-//W3C//DTD SVG 1.1//EN",
+			   (const xmlChar*)"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
+		doc->children = xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL);
+		xmlNsPtr ns = xmlNewNs (doc->children, (const xmlChar*)"http://www.w3.org/2000/svg", NULL);
+		xmlSetNs (doc->children, ns);
+		xmlNewProp (doc->children, (const xmlChar*)"version", (const xmlChar*)"1.1");
+		rect.x0 = floor (rect.x0);
+		rect.y0 = floor (rect.y0);
+		rect.x1 = ceil (rect.x1);
+		rect.y1 = ceil (rect.y1);
+		buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
+		xmlNewProp (doc->children, (const xmlChar*)"width", (const xmlChar*)buf);
+		g_free (buf);
+		buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
+		xmlNewProp (doc->children, (const xmlChar*)"height", (const xmlChar*)buf);
+		g_free (buf);
+		xmlNodePtr node;
+		node = xmlNewDocNode (doc, NULL, (const xmlChar*)"rect", NULL);
+		xmlAddChild (doc->children, node);
+		buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
+		xmlNewProp (node, (const xmlChar*)"width", (const xmlChar*)buf);
+		g_free (buf);
+		buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
+		xmlNewProp (node, (const xmlChar*)"height", (const xmlChar*)buf);
+		g_free (buf);
+		xmlNewProp (node, (const xmlChar*)"stroke", (const xmlChar*)"none");
+		xmlNewProp (node, (const xmlChar*)"fill", (const xmlChar*)"white");
+		if (rect.x0 != 0. || rect.y0 != 0) {
+			node = xmlNewDocNode (doc, NULL, (const xmlChar*)"g", NULL);
+			xmlAddChild (doc->children, node);
+			buf = g_strdup_printf ("translate(%g,%g)", - rect.x0, - rect.y0);
+			xmlNewProp (node, (const xmlChar*)"transform", (const xmlChar*)buf);
+			g_free (buf);
+		} else
+			node = doc->children;
+		g_printable_export_svg (G_PRINTABLE (m_pData->Group), doc, node);
+		setlocale (LC_NUMERIC, old_num_locale);
+		g_free (old_num_locale);
+		xmlIndentTreeOutput = true;
+		xmlKeepBlanksDefault (0);
+		xmlSaveFormatFile (filename, doc, true);
+		xmlFreeDoc (doc);
 	} else {
 		gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
 		GdkDrawable *drawable = GDK_DRAWABLE (gtk_widget_get_parent_window (m_pWidget));
@@ -817,8 +866,10 @@ void gcpView::ExportImage (const char* filename, const char* type)
 		buf.rect.y1 = (int) ceil (rect.y1);
 		buf.buf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 		buf.bg_color = 0xffffff;
+		buf.is_buf = 1;
 		(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &buf);
 		gdk_pixbuf_save (pixbuf, filename, type, NULL, NULL);
 		g_object_unref (pixbuf);
 	}
+	m_pData->ShowSelection (true);
 }
